/*
 * It tries to get the current R command by looking backwards
 * and forwards from the current caret position
 * and returns it. If no command is found
 * nil will be returned. If a selection is given it
 * returns the selection instead.
 */
-(NSString*)getCurrentCommandForCaretPosition
{
	
	NSRange curRange = [self selectedRange];
	
	// if selection is given return it
	if (curRange.length > 0) {
		return([[self string] substringWithRange:curRange]);
	}
	
	NSRange curLineRange = [[self string] lineRangeForRange: NSMakeRange(curRange.location, 0)];
	
	// correct location to get a command if a multi-line code snippet was pasted
	// into the console and the command begins really at the beginning of a line:
	// Example after pasting and looking for “mean”:
	/*
	 > a <- c(
	 mean(2)
	 )
	 */
	curLineRange.location -= 1; 

	unsigned long loc = curLineRange.location;
	SLog(@"RTextView: getCurrentCommandForCaretPosition: current line:%@", [[self string] substringWithRange:curLineRange]);
	
	//look before caret
	unsigned long pos = curRange.location - 1; // caret position
	BOOL skip = NO;       // while loop terminator
	BOOL firstLoop = YES; // to get first while loop
	while(loc < pos) {
		unichar c;
		@try { c = [[self string] characterAtIndex:pos]; }
		@catch (id ae) { skip = YES; break;	}
		if (firstLoop && c == '(') { // ignore ( just before caret 
			pos -= 1; curRange.location -= 1;
		} else {
			switch(c) {
				case ' ':
				case '(':
				case '-':
				case '=':
				case ',':
				case ';':
				case '[':
					skip = YES;
					break;
			}
		}
		if(firstLoop) firstLoop = NO;
		if(skip) break;
		pos -= 1;
	}
	unsigned long wordStart = pos+1;
	// look after caret
	pos = curRange.location;
	skip = NO;
	unsigned long endloc = curLineRange.location+curLineRange.length;
	while(endloc >= pos) {
		unichar c;
		@try { c = [[self string] characterAtIndex:pos]; }
		@catch (id ae) { skip = YES; break;	}
		switch(c) {
			case ' ':
			case '(':
			case ',':
			case ')':
			case ']':
			case '[':
			case '@':
				skip = YES;
				break;
		}
		if(skip) break;
		pos += 1;
	}
	unsigned long wordEnd = pos;
	
	NSRange wordRange = NSMakeRange(wordStart, wordEnd - wordStart);
	SLog(@"RTextView: getCurrentCommandForCaretPosition: found range: %d,%d", wordStart, wordEnd);
	if(wordRange.length < 1) {
		SLog(@"RTextView: getCurrentCommandForCaretPosition: found word:<nil>");
		return(nil);
	}
	SLog(@"RTextView: getCurrentCommandForCaretPosition: found word:%@", [[self string] substringWithRange:wordRange]);
	
	return([[self string] substringWithRange:wordRange]);
}
